/*	$NetBSD: sprintf_chk.c,v 1.6 2009/02/05 05:40:36 lukem Exp $	*/

/*-
 * Copyright (c) 2006 The NetBSD Foundation, Inc.
 * All rights reserved.
 *
 * This code is derived from software contributed to The NetBSD Foundation
 * by Christos Zoulas.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE NETBSD FOUNDATION, INC. AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FOUNDATION OR CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

/*LINTLIBRARY*/

#include <ssp/ssp.h>
#include <stdio.h>
#include <limits.h>
#include <stdarg.h>
#include <ssp/stdio.h>

#undef vsnprintf
#undef vsprintf

int
/*ARGSUSED*/
__sprintf_chk(char * __restrict buf, int flags, size_t slen,
    const char * __restrict fmt, ...)
{
	va_list ap;
	int rv;

        (void) flags;
	va_start(ap, fmt);
	if (slen > (size_t)INT_MAX)
		rv = vsprintf(buf, fmt, ap);
	else {
		if ((rv = vsnprintf(buf, slen, fmt, ap)) >= 0 && (size_t)rv >= slen)
			__chk_fail();
	}
	va_end(ap);

	return rv;
}


#ifdef __LONG_DOUBLE_128__
#ifdef __strong_reference
__strong_reference(__sprintf_chk, __sprintf_chkieee128);
#endif
#endif
